 {
  Demonstration program for ChroMathsome.
  This program shows how to graph mathematical functions in Delphi.
  It also shows how one can implement functions like zooming (in and out) and how to
  scale the graph so that it fits the screen while keeping the correct X-Y coordinates.
  For function names and definitions, look in Calculus.txt.

  Copyright 1996, John Merrit
  All Rights Reserved
  The reader may do with this code whatever he/she likes except...
  'Reproduction of this 'program' for commercial purposes is prohibited.'
  ( I wouldnt mind a little credit... After all, I wrote this program )

  I would appreciate:
  *  Comments, Suggestions, Bug finds (read the comments throughout this code before you call something
     a 'bug') etc.
  *  If you improve this code, please send me a copy of the changes.
  What I do not want is messages starting with "I am new to Delphi...".
  I simply do not have time to support this demoprogram.
  Write to me if you have found a problem, not if you have a problem
  which can be answered by posting to the comp.lang.delphi.components
  newsgroup or in one of the mathematics/computer science/numerical analysis newsgroups.

  Correspondence: John.Merrit@Net4All.BE

  Disclaimer:
  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.                                        }

unit Mathdraw;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls;

type
  avector = array[1..1500] of extended;  { Storage for values of function to draw.}
  { Use 1500 so that the user can resize up to resolution 1500.  }
  TForm1 = class(TForm)
    FunctionEdit: TEdit;
    DrawFunctionButton: TButton;
    Label1: TLabel;
    DrawPad: TPaintBox;
    ClearGraphButton: TButton;
    Bevel1: TBevel;
    ZoomInButton: TButton;
    ZoomOutButton: TButton;
    Label2: TLabel;
    procedure DrawFunctionButtonClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ClearGraphButtonClick(Sender: TObject);
    procedure ZoomInButtonClick(Sender: TObject);
    procedure ZoomOutButtonClick(Sender: TObject);
    procedure DrawPadMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure DrawPadPaint(Sender: TObject);

  private
    { Private declarations }
    procedure PlotGraph;
    procedure DrawAxes;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  ScaleY, ZoomFactor: integer;

implementation

uses Calculus;
{$R *.DFM}

procedure TForm1.DrawAxes;
var i, x, y: integer;
begin
     x:= DrawPad.width;
     y:= DrawPad.height;
     with DrawPad.Canvas do
     begin
     Pen.Color := clred;
     for i:= 1 to x - 1 do
     begin
     moveto(i,Round(y/2));
     lineto(i+1,Round(y/2));
     end;
     for i:= 1 to y - 1 do
     begin
     moveto(Round(x/2),i);
     lineto(Round(x/2),i+1);
     end;
     end;
end;

procedure TForm1.PlotGraph;
var i,x,y: integer;
    fx: extended;
    v:^avector;
    UserFunc: string;
begin
     new(v);
     try
     UserFunc:= FunctionEdit.Text;
     { procedure PrepFunction adds closing brackets (')') if nescessary. }
     PrepFunction(UserFunc);
     x:= DrawPad.width;
     y:= DrawPad.height;
     try
     for i:= 1 to x - 1 do
     begin
     { x:= 0 at 1/2 width drawpad. fx = x- value where function needs to be evaluated. }
     fx:= (i - x/2)/ScaleY;
     v^[i]:= FunctionValue(UserFunc,fx);
     end;
     DrawAxes;
     with DrawPad.Canvas do
     begin
     Pen.Color := clBlue;
     for i:= 1 to x - 2 do
     begin
     moveto(i,Round(y/2 - ScaleY*v^[i]));
     lineto(i+1,Round(y/2 - ScaleY*v^[i+1]));
     end;
     end;
     except on EInvalidOp do showmessage('An error has occured!');
     end;
     finally  { ...make sure the pointer gets freed!!!  }
     dispose(v);
     end;
end;


{ Draw a userdefined function on the drawpad.
  Notice that first step is to calculate the function values (stored in v)
  and next use v to draw the function.
  This approach makes the drawing of complex functions (e.g. exp(cos(sinh(2 + x))) - tan(x))
  more fluently.
  Note that if you resize the drawpad, the graph will automatically adjust itself to the new screensize.
  Take a look at some strange beautifull graphs:
  - sin(x*x*x) ; the function when you start this application
  - cos(x*x*x)
  - 2*exp(-sin(5*x))*cos(3*x) ( a damped oscillating system with sinusodal damping factor?)
  - etc...
  Note that if your function leads to an invalid operation, a message box will inform you so.
  For example: trying to draw the function sqrt(x) without modifying the code will result in
  an invalid operation.
  This is only a demonstration. These modifications are your part!                          }
procedure TForm1.DrawFunctionButtonClick(Sender: TObject);
begin
     PlotGraph;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
     ScaleY:= Round(DrawPad.Height/10);
     ZoomFactor:= 0;
     Label2.Caption:= 'Zoom factor: ' + IntToStr(ZoomFactor);
end;

procedure TForm1.ClearGraphButtonClick(Sender: TObject);
begin
     DrawPad.Refresh;
     DrawAxes;
end;

procedure TForm1.ZoomInButtonClick(Sender: TObject);
begin
     ScaleY:= 2*ScaleY;
     ZoomFactor:= ZoomFactor + 1;
     Label2.Caption:= 'Zoom factor: ' + IntToStr(ZoomFactor);
     DrawPad.Refresh;
     DrawFunctionButtonClick(Sender);
end;

procedure TForm1.ZoomOutButtonClick(Sender: TObject);
begin
     ScaleY:= round(ScaleY/2);
     ZoomFactor:= ZoomFactor - 1;
     Label2.Caption:= 'Zoom factor: ' + IntToStr(ZoomFactor);
     DrawPad.Refresh;
     DrawFunctionButtonClick(Sender);
end;


procedure TForm1.DrawPadMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var dx,dy: real;
begin
   dx:= X - DrawPad.Width/2;
   dy:= DrawPad.Height/2 - Y;
   DrawPad.Canvas.TextOut(5, DrawPad.Height - 15, '(' + FloatToStr(dx/ScaleY) + ';' +
   FloatToStr(dy/ScaleY) + ')     ');
end;

procedure TForm1.FormResize(Sender: TObject);
var OldH, OldW: integer;
begin
    Bevel1.Width:= Form1.Width - 30;
    Bevel1.Height:= Form1.Height - 185;
    DrawPad.Width:= Bevel1.Width - 30;
    DrawPad.Height:= Bevel1.Height - 30;
end;

procedure TForm1.DrawPadPaint(Sender: TObject);
begin
    PlotGraph;
end;

end.
